from django.utils.translation import gettext_lazy as _

undefined = {
    "code": "undefined",
    "en": _("undefined exception code"),
}

tag_100_1 = {
    "code": "tag_100_1",
    "en": _("Tag already exists. name: {name}"),
}

tag_100_2 = {
    "code": "tag_100_2",
    "en": _("Tag not found: {name}"),
}

tag_100_3 = {
    "code": "tag_100_3",
    "en": _("Key not found."),
}

tag_100_4 = {
    "code": "tag_100_4",
    "en": _("Object ids not found."),
}



import argparse
import os
import sys

import django
from django.conf import settings
from django.test.utils import get_runner

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Run the tests for dj-flexi-tag")
    parser.add_argument("-v", "--verbose", action="store_true", help="Verbose output")
    parser.add_argument(
        "-i", "--interactive", action="store_true", help="Interactive mode"
    )
    args = parser.parse_args()

    os.environ["DJANGO_SETTINGS_MODULE"] = "flexi_tag.tests.test_settings"

    if not args.interactive:
        os.environ["DJANGO_ALLOW_ASYNC_UNSAFE"] = "true"
        os.environ["DJANGO_DATABASE_YES_TO_ALL"] = "true"

    django.setup()

    verbosity_level = 2 if args.verbose else 1

    TestRunner = get_runner(settings)
    test_runner = TestRunner(verbosity=verbosity_level, interactive=args.interactive)

    failures = test_runner.run_tests(["flexi_tag.tests"])
    sys.exit(bool(failures))



import importlib
import inspect
import os
import sys
from importlib import import_module

from django.apps import apps
from django.core.management import call_command
from django.core.management.base import BaseCommand
from django.template import Context, Template

from flexi_tag.utils.models import FlexiTagMixin


class Command(BaseCommand):
    help = "Generate tag models for all models that inherit from FlexiTagMixin"

    def add_arguments(self, parser):
        parser.add_argument(
            "--dry-run",
            action="store_true",
            help="Show what would be generated without creating files",
        )
        parser.add_argument(
            "--force-reload",
            action="store_true",
            help="Force reload of all app models to detect recent changes",
        )

    def handle(self, **options):
        dry_run = options.get("dry_run", False)
        force_reload = options.get("force_reload", False)

        # Clear Django's app registry cache to ensure we detect new FlexiTagMixin models
        self.stdout.write("Detecting FlexiTagMixin models...")
        apps.clear_cache()

        if force_reload:
            self.stdout.write("Force reloading models modules...")
            for app_config in apps.get_app_configs():
                if hasattr(app_config, "models_module") and app_config.models_module:
                    try:
                        # Force reload the models module
                        importlib.reload(app_config.models_module)
                    except (ImportError, AttributeError):
                        # Some apps might not have models, that's OK
                        pass

        # Group models by app
        models_by_app = {}

        for app_config in apps.get_app_configs():
            app_models = []
            for model in app_config.get_models():
                if issubclass(model, FlexiTagMixin) and model is not FlexiTagMixin:
                    app_models.append(model)

            if app_models:
                models_by_app[app_config] = app_models

        # Template for the generated model file (supports multiple models)
        model_template = Template(
            """
# This file is auto-generated by the generate_tag_models command via "flexi-tag" package.
# Do not edit manually.

from django.db import models
from flexi_tag.utils.compat import JSONField, GinIndex

{% for model_data in models %}
class {{ model_data.model_name }}Tag(models.Model):
    instance = models.OneToOneField(
        "{{ model_data.app_label }}.{{ model_data.model_name }}",
        on_delete=models.CASCADE,
        primary_key=True,
    )
    tags = JSONField(default=list)

    class Meta:
        indexes = [GinIndex(fields=["tags"])]

    def __str__(self):
        return "Tags for {}".format(self.instance)

{% endfor %}
"""
        )

        for app_config, models in models_by_app.items():
            # Prepare context data for all models in this app
            models_data = []
            tag_model_names = []

            for model in models:
                model_name = model.__name__
                app_label = model._meta.app_label

                models_data.append(
                    {
                        "model_name": model_name,
                        "app_label": app_label,
                        "model_lower_name": model_name.lower(),
                    }
                )
                tag_model_names.append("{}Tag".format(model_name))

            context = Context({"models": models_data})
            model_content = model_template.render(context)

            # Get the directory for this app (using the first model to determine location)
            first_model = models[0]
            model_module = import_module(first_model.__module__)
            model_dir = os.path.dirname(inspect.getfile(model_module))
            file_path = os.path.join(model_dir, "flexi_generated_model.py")

            if dry_run:
                self.stdout.write("Would create file: {}".format(file_path))
                self.stdout.write(model_content)
                self.stdout.write("-" * 40)
            else:
                # Write all models for this app in one operation
                with open(file_path, "w") as f:
                    f.write(model_content)

                # Update imports in models.py
                models_file_path = os.path.join(model_dir, "models.py")
                if os.path.exists(models_file_path):
                    self._update_imports(models_file_path, tag_model_names)

                for model_data in models_data:
                    self.stdout.write(
                        self.style.SUCCESS(
                            "Created tag model for {app_label}.{model_name} at {file_path}".format(
                                app_label=model_data["app_label"],
                                model_name=model_data["model_name"],
                                file_path=file_path,
                            )
                        )
                    )

        try:
            from django.contrib.postgres.indexes import GinIndex  # noqa
        except ImportError:
            self.stdout.write(
                self.style.WARNING(
                    "Important Note: GinIndex is not available. You need to add it manually with migrations."
                )
            )

        if dry_run:
            self.stdout.write("Dry run completed. No files were created.")
        else:
            self.stdout.write(
                self.style.SUCCESS("Successfully generated all tag models!")
            )
            is_test = (
                "test" in sys.argv
                or os.environ.get("DJANGO_SETTINGS_MODULE")
                == "flexi_tag.tests.test_settings"
            )

            if not is_test:
                self.stdout.write("Running makemigrations command...")
                call_command("makemigrations")
                self.stdout.write(
                    self.style.SUCCESS("Successfully created migrations!")
                )
            else:
                self.stdout.write("Skipping makemigrations in test environment.")

    def _update_imports(self, models_file_path, tag_model_names):
        """Update the imports in models.py to include all tag models."""
        with open(models_file_path, "r") as f:
            models_content = f.read()

        import_pattern = "from .flexi_generated_model import"
        lines = models_content.splitlines()
        existing_import_line = None
        last_import_index = -1

        # Find existing import line and last import line
        for i, line in enumerate(lines):
            if line.startswith(import_pattern):
                existing_import_line = line
                last_import_index = i
            elif line.startswith("import ") or line.startswith("from "):
                if existing_import_line is None:
                    last_import_index = i

        if existing_import_line:
            # Replace existing import with all tag models
            import_line = "{import_pattern} {tag_model_names}  # noqa".format(
                import_pattern=import_pattern,
                tag_model_names=", ".join(tag_model_names),
            )
            lines[last_import_index] = import_line
        else:
            # Add new import after last import
            if last_import_index >= 0:
                import_line = "\n\nfrom .flexi_generated_model import {tag_model_names}  # noqa".format(
                    tag_model_names=", ".join(tag_model_names)
                )
                lines.insert(last_import_index + 1, import_line)

        updated_content = "\n".join(lines)
        with open(models_file_path, "w") as f:
            f.write(updated_content)



from django.apps import AppConfig


class FlexiTagConfig(AppConfig):
    """
    Django application configuration for Flexi Tag.

    Flexi Tag provides a flexible, service-only tagging system for Django
    that avoids manager conflicts and provides powerful QuerySet composition.
    """

    name = "flexi_tag"
    verbose_name = "Flexi Tag"
    default_auto_field = "django.db.models.BigAutoField"

    def ready(self):
        """
        Application initialization.

        The service-only architecture requires no special initialization
        as it doesn't rely on model metaclasses or custom managers.
        """
        pass



try:
    from unittest import mock
except ImportError:
    import mock

from django.db import models
from django.db.models import QuerySet
from django.test import TestCase

from flexi_tag.exceptions import TagNotFoundException, TagValidationException
from flexi_tag.tests.models import ServiceTestModel, ServiceTestModelTag
from flexi_tag.utils.service import TaggableService


class TaggableServiceTestCase(TestCase):
    def setUp(self):
        self.service = TaggableService()
        self.test_model = mock.MagicMock(spec=ServiceTestModel)
        self.test_model._meta.app_label = "tests"
        self.test_model.__class__.__name__ = "ServiceTestModel"

    def _setup_mock_tag_model(self, mock_get_tag_model_class):
        mock_tag_model = mock.MagicMock()
        mock_get_tag_model_class.return_value = mock_tag_model
        return mock_tag_model

    def _setup_mock_tag_instance(self, mock_tag_model, tags=None):
        mock_tag_instance = mock.MagicMock()
        mock_tag_instance.tags = tags if tags is not None else []
        mock_tag_model.objects.get_or_create.return_value = (mock_tag_instance, True)
        return mock_tag_instance

    def _setup_filter_with_instance(self, mock_tag_model, mock_tag_instance):
        mock_filter = mock.MagicMock()
        mock_filter.first.return_value = mock_tag_instance
        mock_tag_model.objects.filter.return_value = mock_filter

    @mock.patch("flexi_tag.utils.service.apps.get_model")
    def test_get_tag_model_class(self, mock_get_model):
        mock_get_model.return_value = ServiceTestModelTag

        result = TaggableService._TaggableService__get_tag_model_class(self.test_model)

        mock_get_model.assert_called_once_with("tests", "ServiceTestModelTag")
        self.assertEqual(result, ServiceTestModelTag)

    @mock.patch("flexi_tag.utils.service.apps.get_model")
    def test_get_tag_model_class_not_found(self, mock_get_model):
        mock_get_model.side_effect = LookupError()

        with self.assertRaises(ValueError) as context:
            TaggableService._TaggableService__get_tag_model_class(self.test_model)

        self.assertIn(
            "Tag model for tests.ServiceTestModel not found", str(context.exception)
        )
        self.assertIn(
            "Did you run 'python manage.py generate_tag_models'", str(context.exception)
        )

    def test_tag_key_validation(self):
        self.assertTrue(TaggableService._TaggableService__validate_tag_key("valid_key"))

        invalid_keys = [123, True, None, [], {}]
        for invalid_key in invalid_keys:
            with self.assertRaises(ValueError) as context:
                TaggableService._TaggableService__validate_tag_key(invalid_key)
            self.assertEqual(str(context.exception), "Tag key must be a string.")

    @mock.patch(
        "flexi_tag.utils.service.TaggableService._TaggableService__get_tag_model_class"
    )
    def test_add_tag(self, mock_get_tag_model_class):
        mock_tag_model = self._setup_mock_tag_model(mock_get_tag_model_class)
        mock_tag_instance = self._setup_mock_tag_instance(mock_tag_model)

        result = self.service.add_tag(self.test_model, "test_tag")

        mock_get_tag_model_class.assert_called_once_with(self.test_model)
        mock_tag_model.objects.get_or_create.assert_called_once_with(
            instance=self.test_model
        )
        self.assertEqual(mock_tag_instance.tags, ["test_tag"])
        mock_tag_instance.save.assert_called_once_with(update_fields=["tags"])
        self.assertEqual(result, self.test_model)

    @mock.patch(
        "flexi_tag.utils.service.TaggableService._TaggableService__get_tag_model_class"
    )
    def test_add_tag_already_exists(self, mock_get_tag_model_class):
        mock_tag_model = self._setup_mock_tag_model(mock_get_tag_model_class)
        mock_tag_instance = mock.MagicMock()
        mock_tag_instance.tags = ["existing_tag"]
        mock_tag_model.objects.get_or_create.return_value = (mock_tag_instance, False)

        with self.assertRaises(TagValidationException):
            self.service.add_tag(self.test_model, "existing_tag")

    @mock.patch(
        "flexi_tag.utils.service.TaggableService._TaggableService__get_tag_model_class"
    )
    def test_bulk_add_tags(self, mock_get_tag_model_class):
        mock_tag_model = self._setup_mock_tag_model(mock_get_tag_model_class)
        mock_tag_instance = self._setup_mock_tag_instance(mock_tag_model)

        result = self.service.bulk_add_tags(self.test_model, ["tag1", "tag2", "tag3"])

        mock_get_tag_model_class.assert_called_once_with(self.test_model)
        mock_tag_model.objects.get_or_create.assert_called_once_with(
            instance=self.test_model
        )
        self.assertEqual(mock_tag_instance.tags, ["tag1", "tag2", "tag3"])
        mock_tag_instance.save.assert_called_once_with(update_fields=["tags"])
        self.assertEqual(result, self.test_model)

    @mock.patch(
        "flexi_tag.utils.service.TaggableService._TaggableService__get_tag_model_class"
    )
    def test_bulk_add_tags_already_exists(self, mock_get_tag_model_class):
        mock_tag_model = self._setup_mock_tag_model(mock_get_tag_model_class)
        mock_tag_instance = mock.MagicMock()
        mock_tag_instance.tags = ["existing_tag"]
        mock_tag_model.objects.get_or_create.return_value = (mock_tag_instance, False)

        with self.assertRaises(TagValidationException):
            self.service.bulk_add_tags(self.test_model, ["new_tag", "existing_tag"])

    @mock.patch(
        "flexi_tag.utils.service.TaggableService._TaggableService__get_tag_model_class"
    )
    def test_bulk_operations_with_many_instances(self, mock_get_tag_model_class):
        mock_tag_model = self._setup_mock_tag_model(mock_get_tag_model_class)

        instance1 = mock.MagicMock()
        instance2 = mock.MagicMock()
        mock_queryset = mock.MagicMock(spec=QuerySet)
        mock_queryset.__getitem__.return_value = instance1
        mock_queryset.iterator.return_value = [instance1, instance2]

        mock_tag_instance1 = mock.MagicMock()
        mock_tag_instance1.tags = []
        mock_tag_instance2 = mock.MagicMock()
        mock_tag_instance2.tags = []

        mock_tag_model.objects.get_or_create.side_effect = [
            (mock_tag_instance1, True),
            (mock_tag_instance2, True),
        ]

        result = self.service.bulk_add_tags_with_many_instances(
            mock_queryset, ["tag1", "tag2"]
        )

        self.assertEqual(mock_get_tag_model_class.call_count, 1)
        self.assertEqual(mock_tag_model.objects.get_or_create.call_count, 2)
        self.assertEqual(mock_tag_instance1.save.call_count, 1)
        self.assertEqual(mock_tag_instance2.save.call_count, 1)
        self.assertEqual(result, mock_queryset)

        mock_get_tag_model_class.reset_mock()
        mock_tag_model.reset_mock()

        mock_tag_model = self._setup_mock_tag_model(mock_get_tag_model_class)
        mock_tag_instance = mock.MagicMock()
        mock_tag_instance.tags = ["existing_tag"]
        mock_tag_model.objects.get_or_create.return_value = (mock_tag_instance, False)

        with self.assertRaises(ValueError):
            self.service.bulk_add_tags_with_many_instances(
                mock_queryset, ["valid_tag", 123]
            )

        mock_get_tag_model_class.reset_mock()
        mock_tag_model.reset_mock()

        mock_tag_model = self._setup_mock_tag_model(mock_get_tag_model_class)
        mock_tag_instance = mock.MagicMock()
        mock_tag_instance.tags = ["existing_tag"]
        mock_tag_model.objects.get_or_create.return_value = (mock_tag_instance, False)

        with self.assertRaises(TagValidationException):
            self.service.bulk_add_tags_with_many_instances(
                mock_queryset, ["existing_tag"]
            )

    @mock.patch(
        "flexi_tag.utils.service.TaggableService._TaggableService__get_tag_model_class"
    )
    def test_remove_tag_success(self, mock_get_tag_model_class):
        mock_tag_model = self._setup_mock_tag_model(mock_get_tag_model_class)
        mock_tag_instance = mock.MagicMock()
        tags_list = ["tag_to_remove", "other_tag"]
        mock_tag_instance.tags = tags_list
        mock_tag_model.objects.filter.return_value.first.return_value = (
            mock_tag_instance
        )

        result = self.service.remove_tag(self.test_model, "tag_to_remove")

        mock_get_tag_model_class.assert_called_once_with(self.test_model)
        mock_tag_model.objects.filter.assert_called_once_with(instance=self.test_model)
        self.assertEqual(len(tags_list), 1)
        self.assertEqual(tags_list[0], "other_tag")
        mock_tag_instance.save.assert_called_once_with(update_fields=["tags"])
        self.assertEqual(result, mock_tag_instance)

    @mock.patch(
        "flexi_tag.utils.service.TaggableService._TaggableService__get_tag_model_class"
    )
    def test_remove_tag_error_cases(self, mock_get_tag_model_class):
        mock_tag_model = self._setup_mock_tag_model(mock_get_tag_model_class)

        mock_filter_none = mock.MagicMock()
        mock_filter_none.first.return_value = None
        mock_tag_model.objects.filter.return_value = mock_filter_none

        with self.assertRaises(TagNotFoundException):
            self.service.remove_tag(self.test_model, "any_tag")

        mock_get_tag_model_class.assert_called_with(self.test_model)
        mock_tag_model.objects.filter.assert_called_with(instance=self.test_model)

        mock_get_tag_model_class.reset_mock()
        mock_tag_model.objects.filter.reset_mock()

        mock_tag_instance = mock.MagicMock()
        mock_tag_instance.tags = ["tag1", "tag2"]
        mock_filter_with_tags = mock.MagicMock()
        mock_filter_with_tags.first.return_value = mock_tag_instance
        mock_tag_model.objects.filter.return_value = mock_filter_with_tags

        with self.assertRaises(TagValidationException):
            self.service.remove_tag(self.test_model, "non_existing_tag")

        mock_get_tag_model_class.assert_called_with(self.test_model)
        mock_tag_model.objects.filter.assert_called_with(instance=self.test_model)
        mock_tag_instance.save.assert_not_called()

    @mock.patch(
        "flexi_tag.utils.service.TaggableService._TaggableService__get_tag_model_class"
    )
    def test_bulk_remove_tags_scenarios(self, mock_get_tag_model_class):
        mock_tag_model = self._setup_mock_tag_model(mock_get_tag_model_class)
        tags_list = ["tag1", "tag2", "tag3"]
        mock_tag_instance = mock.MagicMock()
        mock_tag_instance.tags = tags_list
        mock_tag_model.objects.filter.return_value.first.return_value = (
            mock_tag_instance
        )

        keys = ["tag1", "tag3"]
        result = self.service.bulk_remove_tags(self.test_model, keys)

        mock_get_tag_model_class.assert_called_once_with(self.test_model)
        mock_tag_model.objects.filter.assert_called_once_with(instance=self.test_model)
        self.assertEqual(len(tags_list), 1)
        self.assertEqual(tags_list[0], "tag2")
        mock_tag_instance.save.assert_called_once_with(update_fields=["tags"])
        self.assertEqual(result, mock_tag_instance)

        mock_get_tag_model_class.reset_mock()
        mock_tag_model.reset_mock()

        mock_tag_model = self._setup_mock_tag_model(mock_get_tag_model_class)
        mock_tag_model.objects.filter.return_value.first.return_value = None

        with self.assertRaises(TagNotFoundException):
            self.service.bulk_remove_tags(self.test_model, ["tag1", "tag2"])

        mock_get_tag_model_class.assert_called_with(self.test_model)
        mock_tag_model.objects.filter.assert_called_with(instance=self.test_model)

        mock_get_tag_model_class.reset_mock()
        mock_tag_model.reset_mock()

        mock_tag_model = self._setup_mock_tag_model(mock_get_tag_model_class)
        mock_tag_instance = mock.MagicMock()
        mock_tag_instance.tags = mock.MagicMock()
        mock_tag_instance.tags.__contains__.side_effect = lambda key: key == "tag1"
        mock_tag_model.objects.filter.return_value.first.return_value = (
            mock_tag_instance
        )

        with self.assertRaises(TagValidationException):
            self.service.bulk_remove_tags(self.test_model, ["tag1", "non_existent_tag"])

    @mock.patch(
        "flexi_tag.utils.service.TaggableService._TaggableService__get_tag_model_class"
    )
    def test_bulk_remove_tags_with_many_instances(self, mock_get_tag_model_class):
        mock_tag_model = self._setup_mock_tag_model(mock_get_tag_model_class)

        instance1 = mock.MagicMock()
        instance2 = mock.MagicMock()
        mock_queryset = mock.MagicMock(spec=QuerySet)
        mock_queryset.__getitem__.return_value = instance1
        mock_queryset.iterator.return_value = [instance1, instance2]

        tags_list1 = ["tag1", "tag2"]
        mock_tag_instance1 = mock.MagicMock()
        mock_tag_instance1.tags = tags_list1

        tags_list2 = ["tag1", "tag3"]
        mock_tag_instance2 = mock.MagicMock()
        mock_tag_instance2.tags = tags_list2

        mock_tag_model.objects.filter.return_value.first.side_effect = [
            mock_tag_instance1,
            mock_tag_instance2,
        ]

        result = self.service.bulk_remove_tags_with_many_instances(
            mock_queryset, ["tag1"]
        )

        self.assertEqual(mock_get_tag_model_class.call_count, 1)
        self.assertEqual(mock_tag_model.objects.filter.call_count, 2)

        self.assertEqual(len(tags_list1), 1)
        self.assertEqual(tags_list1[0], "tag2")
        self.assertEqual(len(tags_list2), 1)
        self.assertEqual(tags_list2[0], "tag3")

        self.assertEqual(mock_tag_instance1.save.call_count, 1)
        self.assertEqual(mock_tag_instance2.save.call_count, 1)
        self.assertEqual(result, mock_queryset)

    @mock.patch(
        "flexi_tag.utils.service.TaggableService._TaggableService__get_tag_model_class"
    )
    def test_get_tags_scenarios(self, mock_get_tag_model_class):
        # Scenario 1: Getting existing tags
        mock_tag_model = self._setup_mock_tag_model(mock_get_tag_model_class)
        mock_tag_instance = mock.MagicMock()
        mock_tag_instance.tags = ["tag1", "tag2"]
        mock_tag_model.objects.get.return_value = mock_tag_instance

        result = self.service.get_tags(self.test_model)

        mock_get_tag_model_class.assert_called_once_with(self.test_model)
        mock_tag_model.objects.get.assert_called_once_with(instance=self.test_model)
        self.assertEqual(result, ["tag1", "tag2"])

        mock_get_tag_model_class.reset_mock()
        mock_tag_model.reset_mock()

        # Scenario 2: No tags found (returns empty list)
        mock_tag_model = self._setup_mock_tag_model(mock_get_tag_model_class)
        mock_tag_model.objects.get.return_value = None

        result = self.service.get_tags(self.test_model)

        mock_get_tag_model_class.assert_called_once_with(self.test_model)
        mock_tag_model.objects.get.assert_called_once_with(instance=self.test_model)
        self.assertEqual(result, [])

    @mock.patch("flexi_tag.utils.service.apps.get_model")
    def test_queryset_operations(self, mock_get_model):
        # Mock the model class
        mock_model_class = mock.MagicMock()
        mock_model_class.__name__ = "TestModel"
        mock_model_class._meta.app_label = "tests"

        # Mock the tag model
        mock_tag_model = mock.MagicMock()
        mock_get_model.return_value = mock_tag_model

        # Mock queryset with model attribute and filter method
        mock_queryset = mock.MagicMock()
        mock_queryset.model = mock_model_class
        mock_filtered_queryset = mock.MagicMock()
        mock_queryset.filter.return_value = mock_filtered_queryset
        mock_queryset.exclude.return_value = mock_filtered_queryset
        mock_queryset.prefetch_related.return_value = mock_filtered_queryset

        # Test filter_by_tag
        service = TaggableService()
        result = service.filter_by_tag(mock_queryset, "test_tag")

        mock_get_model.assert_called_with("tests", "TestModelTag")
        mock_queryset.filter.assert_called_once_with(
            testmodeltag__tags__contains=["test_tag"]
        )
        self.assertEqual(result, mock_filtered_queryset)

        # Reset mocks
        mock_queryset.reset_mock()
        mock_get_model.reset_mock()

        # Test exclude_by_tag
        result = service.exclude_by_tag(mock_queryset, "test_tag")

        mock_get_model.assert_called_with("tests", "TestModelTag")
        mock_queryset.exclude.assert_called_once_with(
            testmodeltag__tags__contains=["test_tag"]
        )
        self.assertEqual(result, mock_filtered_queryset)

        # Reset mocks
        mock_queryset.reset_mock()
        mock_get_model.reset_mock()

        # Test with_tags
        result = service.with_tags(mock_queryset)

        mock_get_model.assert_called_with("tests", "TestModelTag")
        mock_queryset.prefetch_related.assert_called_once_with("testmodeltag")
        self.assertEqual(result, mock_filtered_queryset)

    @mock.patch("django.apps.apps.get_model")
    def test_add_and_remove_tag_flow(self, mock_get_model):
        mock_tag_model = mock.MagicMock()
        mock_get_model.return_value = mock_tag_model

        instance = ServiceTestModel(name="Test Integration")

        tags_list = []
        mock_tag_instance = mock.MagicMock()
        mock_tag_instance.tags = tags_list

        mock_tag_model.objects.get_or_create.return_value = (mock_tag_instance, True)
        mock_tag_model.objects.filter.return_value.first.return_value = (
            mock_tag_instance
        )

        self.service.add_tag(instance, "temp_tag")
        self.assertEqual(tags_list, ["temp_tag"])

        self.service.remove_tag(instance, "temp_tag")
        self.assertEqual(tags_list, [])

        mock_get_model.assert_called_with("tests", "ServiceTestModelTag")
        mock_tag_model.objects.get_or_create.assert_called_once_with(instance=instance)
        mock_tag_model.objects.filter.assert_called_once_with(instance=instance)


class ServiceOnlyArchitectureTestCase(TestCase):
    """Test cases for the service-only architecture and FlexiTagMixin behavior."""

    def test_models_have_default_objects_manager(self):
        """Test that models still have the default objects manager."""
        from flexi_tag.tests.models import DefaultManagerTestModel

        # Should have objects manager
        self.assertTrue(hasattr(DefaultManagerTestModel, "objects"))
        self.assertIsInstance(DefaultManagerTestModel.objects, models.Manager)

    def test_custom_manager_preservation(self):
        """Test that custom managers are preserved in the service-only approach."""
        from flexi_tag.tests.models import CustomManager, CustomManagerTestModel

        # Should have objects manager (custom one)
        self.assertTrue(hasattr(CustomManagerTestModel, "objects"))
        self.assertIsInstance(CustomManagerTestModel.objects, CustomManager)

        # Custom manager methods should work
        self.assertTrue(hasattr(CustomManagerTestModel.objects, "active"))
        self.assertTrue(hasattr(CustomManagerTestModel.objects, "inactive"))

    def test_service_filter_methods_work(self):
        """Test that service filter methods work with querysets."""
        from flexi_tag.tests.models import DefaultManagerTestModel

        # Create a queryset
        queryset = DefaultManagerTestModel.objects.all()

        # Service methods should be available and callable
        # Note: These will fail until tag models exist, but they should be callable
        service = TaggableService()
        try:
            service.filter_by_tag(queryset, "test_tag")
        except ValueError as e:
            # Expected error when tag model doesn't exist
            self.assertIn("Tag model for", str(e))

    def test_existing_models_work(self):
        """Test that existing models still work correctly with service approach."""
        from flexi_tag.tests.models import ServiceTestModel, TaggableManagerTestModel

        # Both should have objects manager
        for model in [TaggableManagerTestModel, ServiceTestModel]:
            self.assertTrue(hasattr(model, "objects"))
            self.assertIsInstance(model.objects, models.Manager)

    def test_manager_functionality_works(self):
        """Test that standard managers work for database operations."""
        from flexi_tag.tests.models import (
            CustomManagerTestModel,
            DefaultManagerTestModel,
        )

        # Test default manager model
        instance1 = DefaultManagerTestModel.objects.create(name="test1")
        self.assertEqual(DefaultManagerTestModel.objects.count(), 1)
        self.assertEqual(DefaultManagerTestModel.objects.get(name="test1"), instance1)

        # Test custom manager model
        instance2 = CustomManagerTestModel.objects.create(name="test2", is_active=True)
        instance3 = CustomManagerTestModel.objects.create(name="test3", is_active=False)

        # Test standard manager operations
        self.assertEqual(CustomManagerTestModel.objects.count(), 2)

        # Test custom manager methods
        active_objects = CustomManagerTestModel.objects.active()
        inactive_objects = CustomManagerTestModel.objects.inactive()

        self.assertEqual(active_objects.count(), 1)
        self.assertEqual(inactive_objects.count(), 1)
        self.assertEqual(active_objects.first(), instance2)
        self.assertEqual(inactive_objects.first(), instance3)

    def test_service_methods_available(self):
        """Test that TaggableService has all expected methods."""
        # Instance methods
        service = TaggableService()
        self.assertTrue(hasattr(service, "add_tag"))
        self.assertTrue(hasattr(service, "bulk_add_tags"))
        self.assertTrue(hasattr(service, "remove_tag"))
        self.assertTrue(hasattr(service, "get_tags"))

        # Static methods
        self.assertTrue(hasattr(TaggableService, "filter_by_tag"))
        self.assertTrue(hasattr(TaggableService, "exclude_by_tag"))
        self.assertTrue(hasattr(TaggableService, "with_tags"))
        self.assertTrue(hasattr(TaggableService, "filter_by_tags"))
        self.assertTrue(hasattr(TaggableService, "filter_by_any_tag"))

    def test_multiple_inheritance_scenarios(self):
        """Test complex inheritance scenarios work without managers."""
        from flexi_tag.utils.models import FlexiTagMixin

        # Test model with multiple mixins
        class AnotherMixin(models.Model):
            created_at = models.DateTimeField(auto_now_add=True)

            class Meta:
                abstract = True

        class MultiMixinModel(AnotherMixin, FlexiTagMixin):
            name = models.CharField(max_length=100)

            class Meta:
                app_label = "tests"

        # Should have objects manager
        self.assertTrue(hasattr(MultiMixinModel, "objects"))
        self.assertIsInstance(MultiMixinModel.objects, models.Manager)

    def test_abstract_model_not_affected(self):
        """Test that abstract models work correctly."""
        from flexi_tag.utils.models import FlexiTagMixin

        class AbstractTestModel(FlexiTagMixin):
            name = models.CharField(max_length=100)

            class Meta:
                abstract = True

        # Abstract models should work fine
        self.assertTrue(AbstractTestModel._meta.abstract)

    def test_mixin_itself_is_abstract(self):
        """Test that FlexiTagMixin itself is abstract."""
        from flexi_tag.utils.models import FlexiTagMixin

        self.assertTrue(FlexiTagMixin._meta.abstract)



try:
    from unittest import mock
except ImportError:
    import mock

from django.test import TestCase
from rest_framework.response import Response
from rest_framework.viewsets import ModelViewSet

from flexi_tag.exceptions import (
    ObjectIDsNotDefinedException,
    TagNotDefinedException,
    TagNotFoundException,
)
from flexi_tag.tests.models import TaggableManagerTestModel
from flexi_tag.utils.views import TaggableViewSetMixin


class TestViewSet(TaggableViewSetMixin, ModelViewSet):
    queryset = TaggableManagerTestModel.objects.all()


class TaggableViewSetMixinTestCase(TestCase):
    def setUp(self):
        self.viewset = TestViewSet()
        self.viewset.format_kwarg = None
        self.viewset.taggable_service = mock.MagicMock()
        self.mock_object = mock.MagicMock(spec=TaggableManagerTestModel)
        self.viewset.get_object = mock.MagicMock(return_value=self.mock_object)

        # Mock the model class for the queryset
        self.mock_model_class = mock.MagicMock()
        self.mock_model_class.__name__ = "TaggableManagerTestModel"
        self.mock_model_class._meta.app_label = "tests"

        self.mock_queryset = mock.MagicMock()
        self.mock_queryset.model = self.mock_model_class
        self.mock_filtered_queryset = mock.MagicMock()
        self.mock_queryset.filter.return_value = self.mock_filtered_queryset
        self.viewset.get_queryset = mock.MagicMock(return_value=self.mock_queryset)

        self.viewset.paginate_queryset = mock.MagicMock(return_value=None)

        mock_serializer = mock.MagicMock()
        mock_serializer.data = {"data": "serialized_data"}
        self.viewset.get_serializer = mock.MagicMock(return_value=mock_serializer)

    def _create_mock_request(self, data=None, query_params=None):
        mock_request = mock.MagicMock()
        mock_request.data = data or {}
        mock_request.query_params = query_params or {}
        return mock_request

    def _setup_pagination(self, page=None):
        if page is None:
            page = [self.mock_object]

        self.viewset.paginate_queryset.return_value = page
        paginated_response = Response({"results": {"data": "serialized_data"}})
        self.viewset.get_paginated_response = mock.MagicMock(
            return_value=paginated_response
        )
        return paginated_response

    def test_tag_add_operations(self):
        mock_request = self._create_mock_request(data={"key": "test_tag"})

        response = self.viewset.add_tag(mock_request, pk=1)

        self.viewset.taggable_service.add_tag.assert_called_once_with(
            instance=self.mock_object,
            key="test_tag",
        )
        self.assertEqual(response.status_code, 200)

        mock_request = self._create_mock_request()

        with self.assertRaises(TagNotDefinedException):
            self.viewset.add_tag(mock_request, pk=1)

        mock_request = self._create_mock_request(data={"keys": ["tag1", "tag2"]})

        self.viewset.taggable_service.add_tag.reset_mock()
        response = self.viewset.bulk_add_tag(mock_request, pk=1)

        self.viewset.taggable_service.bulk_add_tags.assert_called_once_with(
            instance=self.mock_object,
            keys=["tag1", "tag2"],
        )
        self.assertEqual(response.status_code, 200)

        mock_request = self._create_mock_request()

        with self.assertRaises(TagNotDefinedException):
            self.viewset.bulk_add_tag(mock_request, pk=1)

    def test_bulk_add_tags_scenarios(self):
        mock_request = self._create_mock_request(
            data={"objects": [1, 2], "keys": ["tag1", "tag2"]}
        )

        response = self.viewset.bulk_add_tags(mock_request)

        self.mock_queryset.filter.assert_called_once_with(id__in=[1, 2])
        self.viewset.taggable_service.bulk_add_tags_with_many_instances.assert_called_once_with(
            instances=self.mock_filtered_queryset,
            keys=["tag1", "tag2"],
        )
        self.assertEqual(response.status_code, 200)

        mock_request = self._create_mock_request(data={"objects": [1, 2]})

        with self.assertRaises(TagNotDefinedException):
            self.viewset.bulk_add_tags(mock_request)

        mock_request = self._create_mock_request(data={"keys": ["tag1", "tag2"]})

        with self.assertRaises(ObjectIDsNotDefinedException):
            self.viewset.bulk_add_tags(mock_request)

    def test_tag_remove_operations(self):
        mock_request = self._create_mock_request(data={"key": "test_tag"})

        response = self.viewset.remove_tag(mock_request, pk=1)

        self.viewset.taggable_service.remove_tag.assert_called_once_with(
            instance=self.mock_object,
            key="test_tag",
        )
        self.assertEqual(response.status_code, 200)

        mock_request = self._create_mock_request()

        with self.assertRaises(TagNotFoundException):
            self.viewset.remove_tag(mock_request, pk=1)

        self.viewset.taggable_service.remove_tag.reset_mock()
        mock_request = self._create_mock_request(data={"keys": ["tag1", "tag2"]})

        response = self.viewset.bulk_remove_tags(mock_request, pk=1)

        self.viewset.taggable_service.bulk_remove_tags.assert_called_once_with(
            instance=self.mock_object,
            keys=["tag1", "tag2"],
        )
        self.assertEqual(response.status_code, 200)

        mock_request = self._create_mock_request()

        with self.assertRaises(TagNotDefinedException):
            self.viewset.bulk_remove_tags(mock_request, pk=1)

    def test_bulk_remove_tags_with_many_instances_scenarios(self):
        mock_request = self._create_mock_request(
            data={"objects": [1, 2], "keys": ["tag1", "tag2"]}
        )

        response = self.viewset.bulk_remove_tags_with_many_instances(mock_request)

        self.mock_queryset.filter.assert_called_once_with(id__in=[1, 2])
        self.viewset.taggable_service.bulk_remove_tags_with_many_instances.assert_called_once_with(
            instances=self.mock_filtered_queryset,
            keys=["tag1", "tag2"],
        )
        self.assertEqual(response.status_code, 200)

        mock_request = self._create_mock_request(data={"objects": [1, 2]})

        with self.assertRaises(TagNotDefinedException):
            self.viewset.bulk_remove_tags_with_many_instances(mock_request)

        mock_request = self._create_mock_request(data={"keys": ["tag1", "tag2"]})

        with self.assertRaises(ObjectIDsNotDefinedException):
            self.viewset.bulk_remove_tags_with_many_instances(mock_request)

    def test_get_tags_scenarios(self):
        mock_request = self._create_mock_request()

        tag_instance = {"tags": ["tag1", "tag2"]}
        self.viewset.taggable_service.get_tags.return_value = tag_instance

        response = self.viewset.get_tags(mock_request, pk=1)

        self.viewset.taggable_service.get_tags.assert_called_once_with(
            instance=self.mock_object,
        )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.data, tag_instance)

        mock_request = self._create_mock_request()

        self.viewset.taggable_service.get_tags.reset_mock()
        self.viewset.taggable_service.get_tags.return_value = None

        response = self.viewset.get_tags(mock_request, pk=1)

        self.viewset.taggable_service.get_tags.assert_called_once_with(
            instance=self.mock_object,
        )
        self.assertEqual(response.status_code, 404)

    @mock.patch("flexi_tag.utils.service.apps.get_model")
    def test_filter_by_tag_scenarios(self, mock_get_model):
        # Mock the tag model
        mock_tag_model = mock.MagicMock()
        mock_get_model.return_value = mock_tag_model
        mock_request = self._create_mock_request(query_params={"key": "test_tag"})

        # Mock TaggableService instance method
        with mock.patch.object(
            self.viewset.taggable_service, "filter_by_tag"
        ) as mock_filter_by_tag:
            mock_filter_by_tag.return_value = self.mock_filtered_queryset

            response = self.viewset.filter_by_tag(mock_request)

            mock_filter_by_tag.assert_called_once_with(
                queryset=self.mock_queryset,
                key="test_tag",
            )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.data, {"data": "serialized_data"})

        mock_request = self._create_mock_request()

        with self.assertRaises(TagNotDefinedException):
            self.viewset.filter_by_tag(mock_request)

        mock_request = self._create_mock_request(query_params={"key": "test_tag"})

        page = [self.mock_object]
        paginated_response = self._setup_pagination(page)

        with mock.patch.object(
            self.viewset.taggable_service, "filter_by_tag"
        ) as mock_filter_by_tag:
            mock_filter_by_tag.return_value = self.mock_filtered_queryset

            response = self.viewset.filter_by_tag(mock_request)

            self.viewset.paginate_queryset.assert_called_with(
                self.mock_filtered_queryset
            )
            self.viewset.get_serializer.assert_called_with(page, many=True)
            self.viewset.get_paginated_response.assert_called_with(
                {"data": "serialized_data"}
            )
            self.assertEqual(response, paginated_response)

    @mock.patch("flexi_tag.utils.service.apps.get_model")
    def test_exclude_by_tag_scenarios(self, mock_get_model):
        # Mock the tag model
        mock_tag_model = mock.MagicMock()
        mock_get_model.return_value = mock_tag_model
        mock_request = self._create_mock_request(query_params={"key": "test_tag"})

        # Mock TaggableService instance method
        with mock.patch.object(
            self.viewset.taggable_service, "exclude_by_tag"
        ) as mock_exclude_by_tag:
            mock_exclude_by_tag.return_value = self.mock_filtered_queryset

            response = self.viewset.exclude_by_tag(mock_request)

            mock_exclude_by_tag.assert_called_once_with(
                queryset=self.mock_queryset,
                key="test_tag",
            )
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.data, {"data": "serialized_data"})

        mock_request = self._create_mock_request()

        with self.assertRaises(TagNotDefinedException):
            self.viewset.exclude_by_tag(mock_request)

        mock_request = self._create_mock_request(query_params={"key": "test_tag"})

        page = [self.mock_object]
        paginated_response = self._setup_pagination(page)

        with mock.patch.object(
            self.viewset.taggable_service, "exclude_by_tag"
        ) as mock_exclude_by_tag:
            mock_exclude_by_tag.return_value = self.mock_filtered_queryset

            response = self.viewset.exclude_by_tag(mock_request)

            self.viewset.paginate_queryset.assert_called_with(
                self.mock_filtered_queryset
            )
            self.viewset.get_serializer.assert_called_with(page, many=True)
            self.viewset.get_paginated_response.assert_called_with(
                {"data": "serialized_data"}
            )
            self.assertEqual(response, paginated_response)



"""
Compatibility module for Django version differences.
"""
import django

# JSONField location changed in Django 3.1
if django.VERSION >= (3, 1):
    from django.db.models import JSONField
else:
    from django.contrib.postgres.fields import JSONField

# GinIndex compatibility for different Django versions
try:
    from django.contrib.postgres.indexes import GinIndex
except ImportError:
    # For older Django versions that don't have GinIndex
    class GinIndex:
        def __init__(self, *args, **kwargs):
            pass



"""
Django Flexi Tag - Flexible tagging system for Django with service-only architecture.

This package provides a flexible tagging system for Django models using a clean
service-only architecture that avoids manager conflicts and provides powerful
QuerySet composition capabilities.

Key Components:
- FlexiTagMixin: Simple abstract model marker for taggable models
- TaggableService: Core service with both instance and static methods
- TaggableViewSetMixin: DRF ViewSet integration for REST APIs

Example Usage:
    from flexi_tag.utils.models import FlexiTagMixin
    from flexi_tag.utils.service import TaggableService

    class Article(FlexiTagMixin):
        title = models.CharField(max_length=200)

    # Add tags to instances
    service = TaggableService()
    service.add_tag(article, 'featured')

    # Filter QuerySets by tags
    featured = TaggableService.filter_by_tag(Article.objects.all(), 'featured')
"""

__version__ = "2.0.0"  # Major version bump for service-only architecture
__author__ = "Akinon"
__license__ = "MIT"

default_app_config = "flexi_tag.apps.FlexiTagConfig"

# Note: Components are not imported here to avoid Django setup issues.
# Import them directly from their modules:
#   from flexi_tag.utils.models import FlexiTagMixin
#   from flexi_tag.utils.service import TaggableService
#   from flexi_tag.utils.views import TaggableViewSetMixin

__all__ = [
    "__version__",
]



from django.db import models

from flexi_tag.utils.compat import JSONField
from flexi_tag.utils.models import FlexiTagMixin


# Custom manager for testing
class CustomManager(models.Manager):
    def active(self):
        return self.filter(is_active=True)

    def inactive(self):
        return self.filter(is_active=False)


# Models from test_model_managers.py
class TaggableManagerTestModel(FlexiTagMixin):
    name = models.CharField(max_length=100)

    class Meta:
        app_label = (
            "tests"  # Keeping this as 'tests' for compatibility with existing tests
        )


# Model from test_service.py
class ServiceTestModel(FlexiTagMixin):
    name = models.CharField(max_length=100)

    class Meta:
        app_label = (
            "tests"  # Keeping this as 'tests' for compatibility with existing tests
        )


# Test model with custom manager (to test manager preservation)
class CustomManagerTestModel(FlexiTagMixin):
    name = models.CharField(max_length=100)
    is_active = models.BooleanField(default=True)

    # Custom manager should be preserved
    objects = CustomManager()

    class Meta:
        app_label = "tests"


# Test model with no explicit manager (should get default objects + tag_objects)
class DefaultManagerTestModel(FlexiTagMixin):
    name = models.CharField(max_length=100)

    class Meta:
        app_label = "tests"


class ServiceTestModelTag(models.Model):
    instance = models.OneToOneField(
        "tests.ServiceTestModel",
        on_delete=models.CASCADE,
        primary_key=True,
    )
    tags = JSONField(default=list)

    class Meta:
        app_label = "tests"



#!/usr/bin/env python
import os
import sys

from setuptools import find_packages, setup

sys.path.insert(0, os.path.dirname(__file__))

try:
    from version import __version__
except ImportError:
    __version__ = "0.0.1"

__author__ = "Akinon"
__license__ = "MIT"
__maintainer__ = "Akinon"
__email__ = "dev@akinon.com"

if sys.version_info[0] == 2:
    from io import open

with open("README.md", "r", encoding="utf-8") as readme:
    long_description = readme.read()

setup(
    name="dj-flexi-tag",
    version=__version__,
    author=__author__,
    author_email=__email__,
    maintainer=__maintainer__,
    maintainer_email=__email__,
    description="Add tags to any model in Django via ModelViewSet",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license=__license__,
    url="https://bitbucket.org/akinonteam/dj-flexi-tag",
    project_urls={
        "Documentation": "https://dj-flexi-tag.readthedocs.io",
        "Source Code": "https://bitbucket.org/akinonteam/dj-flexi-tag",
    },
    packages=find_packages(exclude=["*.tests", "*.tests.*", "tests.*", "tests"]),
    platforms="any",
    zip_safe=False,
    use_scm_version={
        "write_to": "./version.py",
        "write_to_template": '__version__ = "{version}"\n',
    },
    install_requires=[
        "Django>=1.11,<5.1",
        "djangorestframework>=3.4.3,<=4.0",
        "psycopg2-binary",
        "pytz",
        "typing-extensions",
    ],
    extras_require={
        "dev": [
            "black==21.5b1",
            "isort==5.0.4",
            "tox",
            "click==7.1.2",
        ],
        "test": [
            "mock",
            "importlib-metadata>=1.4.0,<5.0; python_version == '3.7'",
        ],
        "docs": [
            "sphinx-rtd-theme==1.2.0",
            "sphinx==5.3.0",
            "docutils>=0.18",
        ],
    },
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Framework :: Django",
        "Framework :: Django :: 1.11",
        "Framework :: Django :: 2.0",
        "Framework :: Django :: 2.1",
        "Framework :: Django :: 2.2",
        "Framework :: Django :: 3.0",
        "Framework :: Django :: 3.1",
        "Framework :: Django :: 3.2",
        "Framework :: Django :: 4.0",
        "Framework :: Django :: 4.1",
        "Framework :: Django :: 4.2",
        "Framework :: Django :: 5.0",
    ],
    python_requires=">=3.5",
)



import os
import shutil
import tempfile
from io import StringIO  # Python 3
from unittest import mock

from django.core.management import call_command
from django.template import Context, Engine, Template
from django.test import TestCase

from flexi_tag.management.commands.generate_tag_models import Command
from flexi_tag.utils.models import FlexiTagMixin


class TagModelGenerationTestCase(TestCase):
    def setUp(self):
        self.test_dir = tempfile.mkdtemp()

        self.model_template_string = """
# This file is auto-generated. Do not edit manually.
# Generated by the generate_tag_models command.

from django.db import models
from flexi_tag.utils.compat import JSONField, GinIndex


class {{ model_name }}Tag(models.Model):
    instance = models.OneToOneField(
        "{{ app_label }}.{{ model_name }}",
        on_delete=models.CASCADE,
        primary_key=True,
    )
    tags = JSONField(default=list)

    class Meta:
        indexes = [GinIndex(fields=["tags"])]

    def __str__(self):
        return "Tags for {}".format(self.instance)"
"""

    def tearDown(self):
        shutil.rmtree(self.test_dir)

    def _create_mock_model(
        self, name="MyModel", app_label="myapp", module="myapp.models"
    ):
        mock_model = mock.MagicMock()
        mock_model.__name__ = name
        mock_model._meta.app_label = app_label
        mock_model.__module__ = module
        mock_model.__bases__ = (FlexiTagMixin,)
        return mock_model


class GenerateTagModelsCommandTestCase(TagModelGenerationTestCase):
    @mock.patch("django.template.Engine.get_default")
    def test_template_content(self, mock_get_default):
        engine = Engine()
        mock_get_default.return_value = engine

        template = Template(self.model_template_string)

        context = Context(
            {
                "model_name": "TaggableManagerTestModel",
                "app_label": "testapp",
                "model_lower_name": "taggablemanagertestmodel",
            }
        )

        rendered_template = template.render(context)

        self.assertIn(
            "class TaggableManagerTestModelTag(models.Model):", rendered_template
        )

    @mock.patch(
        "flexi_tag.management.commands.generate_tag_models.apps.get_app_configs"
    )
    @mock.patch("flexi_tag.management.commands.generate_tag_models.Template")
    def test_command_with_no_flexi_tag_models(
        self, mock_template, mock_get_app_configs
    ):
        mock_template_instance = mock.MagicMock()
        mock_template_instance.render.return_value = "# Generated model content"
        mock_template.return_value = mock_template_instance

        out = StringIO()

        mock_model = mock.MagicMock()
        mock_model.__name__ = "RegularModel"
        mock_model.__bases__ = (object,)  # Not a FlexiTagMixin

        mock_app_config = mock.MagicMock()
        mock_app_config.get_models.return_value = [mock_model]
        mock_get_app_configs.return_value = [mock_app_config]

        call_command("generate_tag_models", stdout=out)

        self.assertIn("Successfully generated all tag models", out.getvalue())

    @mock.patch("flexi_tag.management.commands.generate_tag_models.Template")
    @mock.patch(
        "flexi_tag.management.commands.generate_tag_models.apps.get_app_configs"
    )
    @mock.patch("flexi_tag.management.commands.generate_tag_models.import_module")
    @mock.patch("flexi_tag.management.commands.generate_tag_models.inspect.getfile")
    def test_dry_run_option(
        self,
        mock_getfile,
        mock_import_module,
        mock_get_app_configs,
        mock_template_class,
    ):
        mock_template = mock.MagicMock()
        mock_template.render.return_value = "# Mocked template content"
        mock_template_class.return_value = mock_template

        mock_model = self._create_mock_model(
            name="TaggableManagerTestModel",
            app_label="testapp",
            module="testapp.models",
        )

        mock_app_config = mock.MagicMock()
        mock_app_config.get_models.return_value = [mock_model]
        mock_get_app_configs.return_value = [mock_app_config]

        mock_module = mock.MagicMock()
        mock_import_module.return_value = mock_module
        test_file_path = os.path.join(self.test_dir, "models.py")
        mock_getfile.return_value = test_file_path

        # Run the command with --dry-run
        out = StringIO()
        call_command("generate_tag_models", "--dry-run", stdout=out)

        output = out.getvalue()
        self.assertIn("Would create file:", output)
        self.assertIn("Dry run completed. No files were created.", output)
        self.assertNotIn("Successfully generated all tag models", output)

    @mock.patch("sys.exit")
    def test_command_help(self, mock_exit):
        command = Command()
        command.stdout = StringIO()

        help_text = command.help

        self.assertIn("Generate tag models", help_text)
        self.assertTrue(callable(command.add_arguments))

        parser = mock.MagicMock()
        command.add_arguments(parser)
        self.assertTrue(parser.add_argument.called)

        # Check that both arguments are added
        call_args_list = parser.add_argument.call_args_list
        self.assertEqual(len(call_args_list), 2)

        # Check first argument is --dry-run
        first_args, _ = call_args_list[0]
        self.assertEqual(first_args[0], "--dry-run")

        # Check second argument is --force-reload
        second_args, _ = call_args_list[1]
        self.assertEqual(second_args[0], "--force-reload")

        mock_exit.assert_not_called()


class CustomTagGenerationTestCase(TagModelGenerationTestCase):
    @mock.patch("flexi_tag.management.commands.generate_tag_models.Template")
    @mock.patch(
        "flexi_tag.management.commands.generate_tag_models.apps.get_app_configs"
    )
    @mock.patch("flexi_tag.management.commands.generate_tag_models.import_module")
    @mock.patch("flexi_tag.management.commands.generate_tag_models.inspect.getfile")
    @mock.patch("flexi_tag.management.commands.generate_tag_models.open", create=True)
    def test_multiple_models_generation(
        self,
        mock_open,
        mock_getfile,
        mock_import_module,
        mock_get_app_configs,
        mock_template_class,
    ):
        mock_template = mock.MagicMock()
        mock_template.render.return_value = "# Mocked template content"
        mock_template_class.return_value = mock_template

        mock_model1 = self._create_mock_model(name="Model1", app_label="testapp")
        mock_model2 = self._create_mock_model(name="Model2", app_label="testapp")

        mock_app_config = mock.MagicMock()
        mock_app_config.get_models.return_value = [mock_model1, mock_model2]
        mock_get_app_configs.return_value = [mock_app_config]

        mock_module = mock.MagicMock()
        mock_import_module.return_value = mock_module
        test_file_path = os.path.join(self.test_dir, "models.py")
        mock_getfile.return_value = test_file_path

        mock_file = mock.MagicMock()
        mock_open.return_value.__enter__.return_value = mock_file

        out = StringIO()
        call_command("generate_tag_models", stdout=out)

        # With the fix, we should have only 1 file write (not 2) because both models
        # are written to the same file in a single operation
        self.assertEqual(mock_open.call_count, 1)
        self.assertEqual(mock_file.write.call_count, 1)
        self.assertIn("Successfully generated all tag models", out.getvalue())

    @mock.patch("flexi_tag.management.commands.generate_tag_models.Template")
    @mock.patch(
        "flexi_tag.management.commands.generate_tag_models.apps.get_app_configs"
    )
    @mock.patch("flexi_tag.management.commands.generate_tag_models.import_module")
    @mock.patch("flexi_tag.management.commands.generate_tag_models.inspect.getfile")
    @mock.patch("flexi_tag.management.commands.generate_tag_models.open", create=True)
    def test_model_with_special_name(
        self,
        mock_open,
        mock_getfile,
        mock_import_module,
        mock_get_app_configs,
        mock_template_class,
    ):
        mock_template = mock.MagicMock()
        mock_template.render.return_value = "# Mocked template content"
        mock_template_class.return_value = mock_template

        special_name = "Special_Model_Name-With.Characters$"
        mock_model = self._create_mock_model(name=special_name, app_label="testapp")

        mock_app_config = mock.MagicMock()
        mock_app_config.get_models.return_value = [mock_model]
        mock_get_app_configs.return_value = [mock_app_config]

        mock_module = mock.MagicMock()
        mock_import_module.return_value = mock_module
        test_file_path = os.path.join(self.test_dir, "models.py")
        mock_getfile.return_value = test_file_path

        mock_file = mock.MagicMock()
        mock_open.return_value.__enter__.return_value = mock_file

        out = StringIO()
        call_command("generate_tag_models", stdout=out)

        self.assertEqual(mock_template.render.call_count, 1)

        # With the new fix, the context contains a 'models' list instead of individual fields
        context = mock_template.render.call_args[0][0]
        models_data = context.get("models", [])
        self.assertEqual(len(models_data), 1, "Should have data for one model")

        model_data = models_data[0]
        self.assertEqual(model_data["model_name"], special_name)
        self.assertEqual(model_data["model_lower_name"], special_name.lower())
        self.assertIn("Successfully generated all tag models", out.getvalue())

    @mock.patch("flexi_tag.management.commands.generate_tag_models.Template")
    @mock.patch(
        "flexi_tag.management.commands.generate_tag_models.apps.get_app_configs"
    )
    @mock.patch("flexi_tag.management.commands.generate_tag_models.import_module")
    @mock.patch("flexi_tag.management.commands.generate_tag_models.inspect.getfile")
    @mock.patch("flexi_tag.management.commands.generate_tag_models.open", create=True)
    def test_command_with_non_django_class(
        self,
        mock_open,
        mock_getfile,
        mock_import_module,
        mock_get_app_configs,
        mock_template_class,
    ):
        mock_template = mock.MagicMock()
        mock_template.render.return_value = "# Mocked template content"
        mock_template_class.return_value = mock_template

        mock_model = mock.MagicMock()
        mock_model.__name__ = "NonModelClass"
        mock_model.__bases__ = (FlexiTagMixin,)
        mock_model._meta = mock.MagicMock()
        mock_model._meta.app_label = "testapp"
        mock_model.__module__ = "testapp.models"

        mock_app_config = mock.MagicMock()
        mock_app_config.get_models.return_value = [mock_model]
        mock_get_app_configs.return_value = [mock_app_config]

        mock_module = mock.MagicMock()
        mock_import_module.return_value = mock_module
        test_file_path = os.path.join(self.test_dir, "models.py")
        mock_getfile.return_value = test_file_path

        mock_file = mock.MagicMock()
        mock_open.return_value.__enter__.return_value = mock_file

        out = StringIO()
        call_command("generate_tag_models", stdout=out)

        self.assertIn("Successfully generated all tag models", out.getvalue())


class AutoImportTestCase(TagModelGenerationTestCase):
    def setUp(self):
        super(AutoImportTestCase, self).setUp()
        self.models_content = """
from django.db import models
from flexi_tag.utils.models import FlexiTagMixin

class TaggableManagerTestModel(FlexiTagMixin):
    name = models.CharField(max_length=100)
"""

    @mock.patch("flexi_tag.management.commands.generate_tag_models.Template")
    @mock.patch("flexi_tag.management.commands.generate_tag_models.inspect.getfile")
    @mock.patch("flexi_tag.management.commands.generate_tag_models.import_module")
    @mock.patch(
        "flexi_tag.management.commands.generate_tag_models.apps.get_app_configs"
    )
    def test_auto_import_single_model(
        self,
        mock_get_app_configs,
        mock_import_module,
        mock_getfile,
        mock_template_class,
    ):
        model_dir = os.path.join(self.test_dir, "testapp")
        if not os.path.exists(model_dir):
            os.makedirs(model_dir)
        models_path = os.path.join(model_dir, "models.py")

        with open(models_path, "w") as f:
            f.write(self.models_content)

        mock_template = mock.MagicMock()
        mock_template.render.return_value = "# Generated model content"
        mock_template_class.return_value = mock_template

        mock_model = self._create_mock_model(
            name="TaggableManagerTestModel", app_label="testapp"
        )

        mock_app_config = mock.MagicMock()
        mock_app_config.get_models.return_value = [mock_model]
        mock_get_app_configs.return_value = [mock_app_config]

        mock_import_module.return_value = mock.MagicMock()
        mock_getfile.return_value = os.path.join(model_dir, "__init__.py")

        out = StringIO()
        call_command("generate_tag_models", stdout=out)

        with open(models_path, "r") as f:
            updated_content = f.read()

        self.assertIn(
            "from .flexi_generated_model import TaggableManagerTestModelTag  # noqa",
            updated_content,
        )

    @mock.patch("flexi_tag.management.commands.generate_tag_models.Template")
    @mock.patch("flexi_tag.management.commands.generate_tag_models.inspect.getfile")
    @mock.patch("flexi_tag.management.commands.generate_tag_models.import_module")
    @mock.patch(
        "flexi_tag.management.commands.generate_tag_models.apps.get_app_configs"
    )
    def test_auto_import_multiple_models(
        self,
        mock_get_app_configs,
        mock_import_module,
        mock_getfile,
        mock_template_class,
    ):
        models_content = """
from django.db import models
from flexi_tag.utils.models import FlexiTagMixin

class FirstModel(FlexiTagMixin):
    name = models.CharField(max_length=100)

class SecondModel(FlexiTagMixin):
    title = models.CharField(max_length=200)
"""

        model_dir = os.path.join(self.test_dir, "testapp")
        if not os.path.exists(model_dir):
            os.makedirs(model_dir)
        models_path = os.path.join(model_dir, "models.py")

        with open(models_path, "w") as f:
            f.write(models_content)

        mock_template = mock.MagicMock()
        mock_template.render.return_value = "# Generated model content"
        mock_template_class.return_value = mock_template

        mock_model1 = self._create_mock_model(name="FirstModel", app_label="testapp")
        mock_model2 = self._create_mock_model(name="SecondModel", app_label="testapp")

        mock_app_config = mock.MagicMock()
        mock_app_config.get_models.return_value = [mock_model1, mock_model2]
        mock_get_app_configs.return_value = [mock_app_config]

        mock_import_module.return_value = mock.MagicMock()
        mock_getfile.return_value = os.path.join(model_dir, "__init__.py")

        out = StringIO()
        call_command("generate_tag_models", stdout=out)

        with open(models_path, "r") as f:
            updated_content = f.read()

        self.assertIn("from .flexi_generated_model import", updated_content)
        self.assertIn("FirstModelTag", updated_content)
        self.assertIn("SecondModelTag", updated_content)
        self.assertIn("# noqa", updated_content)


class MultipleModelsFixTestCase(TagModelGenerationTestCase):
    """Test cases for the fix that handles multiple FlexiTagMixin models correctly."""

    @mock.patch("flexi_tag.management.commands.generate_tag_models.Template")
    @mock.patch("flexi_tag.management.commands.generate_tag_models.inspect.getfile")
    @mock.patch("flexi_tag.management.commands.generate_tag_models.import_module")
    @mock.patch(
        "flexi_tag.management.commands.generate_tag_models.apps.get_app_configs"
    )
    def test_multiple_models_same_app_single_file_generation(
        self,
        mock_get_app_configs,
        mock_import_module,
        mock_getfile,
        mock_template_class,
    ):
        """Test that multiple models in the same app generate a single file with all tag models."""
        model_dir = os.path.join(self.test_dir, "testapp")
        if not os.path.exists(model_dir):
            os.makedirs(model_dir)

        # Mock template that supports multiple models
        mock_template = mock.MagicMock()
        expected_content = """
# This file is auto-generated by the generate_tag_models command via "flexi-tag" package.
# Do not edit manually.

from django.db import models
from flexi_tag.utils.compat import JSONField, GinIndex


class FirstModelTag(models.Model):
    instance = models.OneToOneField(
        "testapp.FirstModel",
        on_delete=models.CASCADE,
        primary_key=True,
    )
    tags = JSONField(default=list)

    class Meta:
        indexes = [GinIndex(fields=["tags"])]

    def __str__(self):
        return "Tags for {}".format(self.instance)


class SecondModelTag(models.Model):
    instance = models.OneToOneField(
        "testapp.SecondModel",
        on_delete=models.CASCADE,
        primary_key=True,
    )
    tags = JSONField(default=list)

    class Meta:
        indexes = [GinIndex(fields=["tags"])]

    def __str__(self):
        return "Tags for {}".format(self.instance)

"""
        mock_template.render.return_value = expected_content
        mock_template_class.return_value = mock_template

        # Create mock models for the same app
        mock_model1 = self._create_mock_model(
            name="FirstModel", app_label="testapp", module="testapp.models"
        )
        mock_model2 = self._create_mock_model(
            name="SecondModel", app_label="testapp", module="testapp.models"
        )

        mock_app_config = mock.MagicMock()
        mock_app_config.get_models.return_value = [mock_model1, mock_model2]
        mock_get_app_configs.return_value = [mock_app_config]

        mock_import_module.return_value = mock.MagicMock()
        mock_getfile.return_value = os.path.join(model_dir, "__init__.py")

        out = StringIO()
        call_command("generate_tag_models", stdout=out)

        # Check that template.render was called with context containing both models
        render_call_args = mock_template.render.call_args[0][0]
        models_data = render_call_args.get("models", [])

        self.assertEqual(len(models_data), 2, "Should have data for both models")
        model_names = [model["model_name"] for model in models_data]
        self.assertIn("FirstModel", model_names)
        self.assertIn("SecondModel", model_names)

        # Check that the generated file path is correct
        expected_file_path = os.path.join(model_dir, "flexi_generated_model.py")
        self.assertTrue(os.path.exists(expected_file_path))

        # Check output messages
        output = out.getvalue()
        self.assertIn("Created tag model for testapp.FirstModel", output)
        self.assertIn("Created tag model for testapp.SecondModel", output)

    @mock.patch("flexi_tag.management.commands.generate_tag_models.Template")
    @mock.patch("flexi_tag.management.commands.generate_tag_models.inspect.getfile")
    @mock.patch("flexi_tag.management.commands.generate_tag_models.import_module")
    @mock.patch(
        "flexi_tag.management.commands.generate_tag_models.apps.get_app_configs"
    )
    def test_multiple_apps_with_multiple_models(
        self,
        mock_get_app_configs,
        mock_import_module,
        mock_getfile,
        mock_template_class,
    ):
        """Test that models from different apps generate separate files correctly."""
        # Create directories for two different apps
        app1_dir = os.path.join(self.test_dir, "app1")
        app2_dir = os.path.join(self.test_dir, "app2")
        # Create directories if they don't exist
        if not os.path.exists(app1_dir):
            os.makedirs(app1_dir)
        if not os.path.exists(app2_dir):
            os.makedirs(app2_dir)

        mock_template = mock.MagicMock()
        mock_template.render.return_value = "# Generated model content"
        mock_template_class.return_value = mock_template

        # Create models for app1
        mock_model1_app1 = self._create_mock_model(
            name="App1Model1", app_label="app1", module="app1.models"
        )
        mock_model2_app1 = self._create_mock_model(
            name="App1Model2", app_label="app1", module="app1.models"
        )

        # Create models for app2
        mock_model1_app2 = self._create_mock_model(
            name="App2Model1", app_label="app2", module="app2.models"
        )
        mock_model2_app2 = self._create_mock_model(
            name="App2Model2", app_label="app2", module="app2.models"
        )

        # Mock app configs
        mock_app_config1 = mock.MagicMock()
        mock_app_config1.get_models.return_value = [mock_model1_app1, mock_model2_app1]

        mock_app_config2 = mock.MagicMock()
        mock_app_config2.get_models.return_value = [mock_model1_app2, mock_model2_app2]

        mock_get_app_configs.return_value = [mock_app_config1, mock_app_config2]

        def mock_import_side_effect(module_name):
            return mock.MagicMock()

        def mock_getfile_side_effect(module):
            # Return different paths based on which app we're dealing with
            if hasattr(module, "_mock_name"):
                if "app1" in str(module._mock_name):
                    return os.path.join(app1_dir, "__init__.py")
                elif "app2" in str(module._mock_name):
                    return os.path.join(app2_dir, "__init__.py")
            return os.path.join(app1_dir, "__init__.py")  # Default

        mock_import_module.side_effect = mock_import_side_effect
        mock_getfile.side_effect = mock_getfile_side_effect

        out = StringIO()
        call_command("generate_tag_models", stdout=out)

        # Check that template.render was called twice (once for each app)
        self.assertEqual(mock_template.render.call_count, 2)

        # Check output contains models from both apps
        output = out.getvalue()
        self.assertIn("App1Model1", output)
        self.assertIn("App1Model2", output)
        self.assertIn("App2Model1", output)
        self.assertIn("App2Model2", output)

    @mock.patch("flexi_tag.management.commands.generate_tag_models.Template")
    @mock.patch("flexi_tag.management.commands.generate_tag_models.inspect.getfile")
    @mock.patch("flexi_tag.management.commands.generate_tag_models.import_module")
    @mock.patch(
        "flexi_tag.management.commands.generate_tag_models.apps.get_app_configs"
    )
    def test_dry_run_with_multiple_models(
        self,
        mock_get_app_configs,
        mock_import_module,
        mock_getfile,
        mock_template_class,
    ):
        """Test dry run mode with multiple models shows all models correctly."""
        mock_template = mock.MagicMock()
        mock_template.render.return_value = """
class Model1Tag(models.Model):
    pass

class Model2Tag(models.Model):
    pass
"""
        mock_template_class.return_value = mock_template

        mock_model1 = self._create_mock_model(
            name="Model1", app_label="testapp", module="testapp.models"
        )
        mock_model2 = self._create_mock_model(
            name="Model2", app_label="testapp", module="testapp.models"
        )

        mock_app_config = mock.MagicMock()
        mock_app_config.get_models.return_value = [mock_model1, mock_model2]
        mock_get_app_configs.return_value = [mock_app_config]

        mock_import_module.return_value = mock.MagicMock()
        mock_getfile.return_value = os.path.join(self.test_dir, "__init__.py")

        out = StringIO()
        call_command("generate_tag_models", "--dry-run", stdout=out)

        output = out.getvalue()
        self.assertIn("Would create file:", output)
        self.assertIn("Model1Tag", output)
        self.assertIn("Model2Tag", output)
        self.assertIn("Dry run completed. No files were created.", output)

    def test_update_imports_method_with_multiple_models(self):
        """Test the _update_imports helper method with multiple tag models."""
        from flexi_tag.management.commands.generate_tag_models import Command

        # Create a test models.py file
        models_content = """
from django.db import models
from flexi_tag.utils.models import FlexiTagMixin

class Model1(FlexiTagMixin):
    name = models.CharField(max_length=100)

class Model2(FlexiTagMixin):
    title = models.CharField(max_length=200)
"""
        models_file_path = os.path.join(self.test_dir, "models.py")
        with open(models_file_path, "w") as f:
            f.write(models_content)

        # Test the _update_imports method
        command = Command()
        tag_model_names = ["Model1Tag", "Model2Tag"]
        command._update_imports(models_file_path, tag_model_names)

        # Read the updated content
        with open(models_file_path, "r") as f:
            updated_content = f.read()

        # Check that both imports are added correctly
        self.assertIn(
            "from .flexi_generated_model import Model1Tag, Model2Tag  # noqa",
            updated_content,
        )

    def test_update_imports_method_with_existing_import(self):
        """Test the _update_imports helper method when an import already exists."""
        from flexi_tag.management.commands.generate_tag_models import Command

        # Create a test models.py file with existing import
        models_content = """
from django.db import models
from flexi_tag.utils.models import FlexiTagMixin
from .flexi_generated_model import OldModelTag  # noqa

class Model1(FlexiTagMixin):
    name = models.CharField(max_length=100)

class Model2(FlexiTagMixin):
    title = models.CharField(max_length=200)
"""
        models_file_path = os.path.join(self.test_dir, "models.py")
        with open(models_file_path, "w") as f:
            f.write(models_content)

        # Test the _update_imports method
        command = Command()
        tag_model_names = ["Model1Tag", "Model2Tag"]
        command._update_imports(models_file_path, tag_model_names)

        # Read the updated content
        with open(models_file_path, "r") as f:
            updated_content = f.read()

        # Check that the import is replaced with all new models
        self.assertIn(
            "from .flexi_generated_model import Model1Tag, Model2Tag  # noqa",
            updated_content,
        )
        self.assertNotIn("OldModelTag", updated_content)

    @mock.patch("flexi_tag.management.commands.generate_tag_models.Template")
    @mock.patch("flexi_tag.management.commands.generate_tag_models.inspect.getfile")
    @mock.patch("flexi_tag.management.commands.generate_tag_models.import_module")
    @mock.patch(
        "flexi_tag.management.commands.generate_tag_models.apps.get_app_configs"
    )
    def test_no_overwrite_issue_with_multiple_models(
        self,
        mock_get_app_configs,
        mock_import_module,
        mock_getfile,
        mock_template_class,
    ):
        """Test that multiple models don't overwrite each other in the generated file."""
        model_dir = os.path.join(self.test_dir, "testapp")
        # Create directory if it doesn't exist
        if not os.path.exists(model_dir):
            os.makedirs(model_dir)

        # This test verifies the core fix: that we write once with all models
        # instead of writing multiple times and overwriting

        written_content = []

        def mock_file_write(content):
            written_content.append(content)

        mock_template = mock.MagicMock()
        mock_template.render.return_value = "# Content with both models"
        mock_template_class.return_value = mock_template

        mock_model1 = self._create_mock_model(
            name="FirstModel", app_label="testapp", module="testapp.models"
        )
        mock_model2 = self._create_mock_model(
            name="SecondModel", app_label="testapp", module="testapp.models"
        )

        mock_app_config = mock.MagicMock()
        mock_app_config.get_models.return_value = [mock_model1, mock_model2]
        mock_get_app_configs.return_value = [mock_app_config]

        mock_import_module.return_value = mock.MagicMock()
        mock_getfile.return_value = os.path.join(model_dir, "__init__.py")

        # Mock the file writing to track how many times it's called
        with mock.patch("builtins.open", mock.mock_open()) as mock_file:
            mock_file.return_value.write.side_effect = mock_file_write

            out = StringIO()
            call_command("generate_tag_models", stdout=out)

            # The key assertion: file should be opened for writing only ONCE per app
            # This is the fix - before, it would open and write multiple times
            write_calls = [
                call for call in mock_file.call_args_list if "w" in str(call)
            ]
            self.assertEqual(
                len(write_calls),
                1,
                "File should be opened for writing only once per app, not once per model",
            )

            # Template should be rendered only once with context for all models
            self.assertEqual(
                mock_template.render.call_count,
                1,
                "Template should be rendered once with all models",
            )

            # Check that the context contains both models
            render_context = mock_template.render.call_args[0][0]
            models_data = render_context.get("models", [])
            self.assertEqual(len(models_data), 2, "Context should contain both models")



from django.db import models


class FlexiTagMixin(models.Model):
    """
    Mixin for tag model generation.

    This mixin is used by the generate_tag_models command to identify models
    that should have a corresponding tag model generated.

    The mixin doesn't add any fields to the model itself. It serves as a marker
    to indicate that a model should have tag functionality.

    The tag model will be generated as:

    class YourModelTag(models.Model):
        instance = models.OneToOneField(
            "app_label.YourModel",
            on_delete=models.CASCADE,
            primary_key=True,
        )
        tags = JSONField(default=list)  # Uses the appropriate JSONField for your Django version

        class Meta:
            indexes = [GinIndex(fields=["tags"])]

    Usage Examples:

    1. Basic usage:
        class YourModel(FlexiTagMixin):
            name = models.CharField(max_length=100)

    After defining your models, run:
        python manage.py generate_tag_models

    This will create flexi_generated_model.py files in the same directories as your models.

    To use the tags functionality, use the TaggableService:
        from flexi_tag.utils.service import TaggableService

        # Add a tag
        TaggableService().add_tag(instance, 'key')

        # Filter by tag (preserves existing queryset filters!)
        queryset = YourModel.objects.filter(is_active=True)
        tagged_items = TaggableService.filter_by_tag(queryset, 'important')

        # Get all tags
        tags_list = TaggableService().get_tags(instance)

        # Remove a tag
        TaggableService().remove_tag(instance, 'key')
    """

    class Meta:
        abstract = True
